#include <vector>
#include <cstdio>
#include <cstdlib>

using std::printf;
using std::putchar;
using std::fopen;
using std::fread;
using std::fclose;

#include "atoms.hh"
#include "level.hh"
#include "pairs.hh"
#include "handling.hh"
#include "rom.hh"

static void CreateRoom(Room& room, const Char* outmap)
{
    // Convert to blocks
    for(unsigned y=0; y<8; ++y)
        for(unsigned x=0; x<8; ++x)
        {
            Block tmp;
            tmp.c[0] = outmap[(y*2+0)*16 + (x*2+0)];
            tmp.c[1] = outmap[(y*2+1)*16 + (x*2+0)];
            tmp.c[2] = outmap[(y*2+0)*16 + (x*2+1)];
            tmp.c[3] = outmap[(y*2+1)*16 + (x*2+1)];
            room.blocks[x*8+y] = Blocks.Get(tmp);
        }
}

static unsigned WriteLevel(Level& target, unsigned& offset, unsigned layout_begin)
{
    unsigned addr_layout = layout_begin + 1;
    
    unsigned byte = 0x00;

    for(unsigned a=0;
        a < MaxRooms && target.rooms[a].cont_dir != Last;
        ++a)
    {
        Screen& screen = target.rooms[a];

        target.CreateChar(screen);
        
        Room room;
        CreateRoom(room, screen.outmap);
        
        unsigned room_id = Rooms.Get(room);
        
        ROM[BaseAddr + Addr_Order + offset++] = room_id;
        
        if(screen.cont_dir & Shutter) byte |= 0x20;
        if(screen.cont_dir & Down)    byte |= 0x40;
        if(screen.cont_dir & Up)      byte |= 0x80;
        
        if(byte & 0xE0)
        {
            ROM[addr_layout++] = byte;
            byte = 0;
        }
        else
        {
            ROM[addr_layout] = byte;
            ++byte;
        }
    }
    return addr_layout - layout_begin;
}

static void WritePair(const LevelPairData& pair)
{
    BaseAddr = pair.BeginAddr;
    
    Rooms.clear();
    Blocks.clear();
    
    unsigned offset = 0;
    
    ROM[Addr_OrderOffsetTable + pair.level1.LevelNumber] = offset;
    pair.level1.ComposeObjects();
    pair.level1.WritePalette(BaseAddr + Addr_Palette1);
    pair.level1.WriteActives(offset, BaseAddr + Addr_Actives1, BaseAddr + Addr_Actives2);
    pair.level1.WriteEnemies(offset, Addr_EnemyPointers + pair.level1.LevelNumber*2,
                             Addr_EnemyData,
                             Addr_EnemyEnd);
    
    unsigned lay = BaseAddr + Addr_Layout;
    lay += WriteLevel(pair.level1, offset, lay);
    ROM[Addr_BossOffsetTable + pair.level1.LevelNumber] = offset-1;

/*    if(&pair.level1 != &pair.level2)
    {
        pair.level2.ComposeObjects();
        ROM[Addr_OrderOffsetTable + pair.level2.LevelNumber] = offset;
        pair.level2.WritePalette(BaseAddr + Addr_Palette2);
        pair.level2.WriteActives(offset, BaseAddr + Addr_Actives2, BaseAddr + Addr_ActivesEnd);
        pair.level2.WriteEnemies(offset, Addr_EnemyPointers + pair.level2.LevelNumber*2,
                                 Addr_EnemyData,
                                 Addr_EnemyEnd);
        
        lay += WriteLevel(pair.level2, offset, lay);
        ROM[Addr_BossOffsetTable + pair.level2.LevelNumber] = offset-1;
    }
*/
    if(offset > 48)
    {
        fprintf(stderr, "Error: Too many levels in a pair (%u)!\n", offset);
    }
}

int main(void)
{
// 3A = wily3 boss
// 3B = activate enemy
// 3C = small ball
// 3D = small weapon
// 3E = small energy
// 3F = big weapon power
// 44 = end level
// 45 = ?
// 49 = random powerup

    WritePair(pair1);/*
    WritePair(pair2);
    WritePair(pair3);
    WritePair(pair4);
    WritePair(pair5);
    WritePair(pair6);*/
}
