/*
ti pa

61.0 = wall
60.0 = walkable surface
65.0 = torch (right)
6E.0 = torch (up)
77.0 = wall detail (best used in horizontal stretches)


28.2 = background: red
29.2 = background: red - shadow
22.2 = background: red - window
23.2 = background: red - window shadow
2B.2 = background: red - window hole

00.0 = background: black

27.2         = detail
26.2         = detail in detail (small)
28.2         = detail in detail (big)

E4.0 = background: fence

83.0 = ladder

30.3 = lava surface
38.3 = lava

00.3 36.3
3F.3 3E.3 = lava flow corner /

6C.0 6D.0 = vert lava tunnel

74.0 75.0
7C.0 7D.0 = lava flow junction

73.0
7B.0 = horiz lava tunnel

51.0 52.0 53.0 = wily ornament
10.0 = wily door




*/
static void FiremanAscii(Screen& s)
{
    for(unsigned y=0; y<16; ++y)
        for(unsigned x=0; x<16; ++x)
        {
            unsigned char c = s.outmap[y*16+x].tile;
            char ch = '?';
            /* Classify types.
             *
             *
             *
             */
            switch(c)
            {
                // air:
                case 0x00: // black bg
                case 0x26 ... 0x27: // detail
                case 0x2F: // detail in detail
                    ch = '.';
                    break;
                
                case 0xE4: // fence
                    ch = '=';
                    break;

                case 0x28:
                case 0x29:
                case 0x22:
                case 0x23:
                case 0x2B: // red walls
                    ch = ' ';
                    break;

                case 0x61: // wall
                case 0x60: // walkable wall
                case 0x51: case 0x52: case 0x53: // wily signature
                case 0x77: // wall detail
                    ch = '#';
                    break;
                
                case 0x6C: case 0x6D: // lava <->
                case 0x74: case 0x75: 
                case 0x7C: case 0x7D: // lava junction
                case 0x73: case 0x7B: // lava up/down
                    ch = '%';
                    break;
                
                case 0x36:
                case 0x3F: case 0x3E: // lava flow corner
                    ch = '/';
                    break;
                
                case 0x83:
                    ch = Ascii_Ladder;
                    break;
                
                case 0x10:
                    ch = Ascii_WilyDoor;
                    break;
                
                case 0x30:
                case 0x38:
                    ch = Ascii_Spike; // really lava
                    break;
                
                case 0x65: // torch right
                    ch = '>';
                    break;
                case 0x6E: // torch up
                    ch = 'T';
                    break;
                // no spikes or gutsblocks
            }
            
            s.ascii[y*32+x] = ch;
            s.ascii[y*32+x+16] = ' ';
            //printf("%02X",c);
        }
}

static void FiremanChar(Screen& s)
{
    unsigned randseed = 0;
    for(unsigned y=0; y<16; ++y)
        for(unsigned x=0; x<16; ++x)
            randseed += s.ascii[y*32+x];
    std::srand(randseed);
    
    char decorations[16*16] = { 0 };
    
    // Pass 1: On black ranges that are limited in length,
    // create detail-lines
    for(unsigned y=0; y<16; ++y) 
    {
        bool blank = true;
        unsigned blankbegin = 0;
        for(unsigned x=0; x<16; ++x)
        {
            if(s.ascii[y*32+x]=='.')
            {
                if(!blank) blankbegin = x;
                blank=true;
            }
            else
            {
                if(blankbegin > 0 && rand()%10 == 0)
                {
                    for(unsigned a=blankbegin; a<x; ++a)
                        decorations[y*16+a] = 1;
                }
                blank = false;
            }
        }
    }
    // Pass 2: On red ranges, create some windows
    

    
    // Convert to tiles
    for(unsigned y=0; y<16; ++y) 
        for(unsigned x=0; x<16; ++x)
        {
            unsigned char c   = 0;
            unsigned char pal = 0;
            
            switch(s.ascii[y*32+x])
            {
                case '.':
                {
                    c   = 0x00; // black background
                    pal = 0x00;
                    
                    break;
                }
                case ' ':
                    if(y > 0 &&
                        (s.ascii[(y-1)*32+x] != '.'
                      && s.ascii[(y-1)*32+x] != ' ')
                      )
                        c = 0x25; // shadow
                    else
                    {
                        switch(decorations[y*16+x])
                        {
                            case 1:
                                c = 0x2C; // inset decoration
                                break;
                            case 2:
                                c = 0x35;
                                if(x<16
                                && decorations[y*16+x+1]==2
                                && (x==0 || decorations[y*16+x-1]!=2))
                                    c = 0x34; // beginning

                                if(x>0  && s.ascii[y*32+x-1] == ' '
                                && x<15 && s.ascii[y*32+x+1] == ' ')
                                {
                                    break;
                                }
                                // passthru
                            case 0:
                                c = 0x26; // no decoration
                        }
                    }
                    pal = 0x02;
                    break;
                case '#':
                    if(y > 0 &&
                        (s.ascii[(y-1)*32+x] != '#')
                      )
                        c = 0x62; // step
                    else
                        c = 0x68 + ((x+y) & 1);
                    pal = 0x00;
                    break;
                case Ascii_Ladder:
                    c   = 0x82; // ladder
                    pal = 0x00;
                    break;
                case Ascii_Spike:
                    c   = 0xC4; // spike
                    if(y>0 && s.ascii[(y-1)*32+x] == ' ')
                        pal = 0x03;
                    else
                        pal = 0x01;
                    break;
                case Ascii_Guts:
                    c   = 0x38 + (x&1) + (y&1)*2; // gutsblock
                    pal = 0x00;
                    break;
                case Ascii_WilyDoor:
                    c   = 0x10;
                    pal = 0x00;
                    break;
                default:
                    printf("? '%c'\n", s.ascii[y*32+x]);
            }
            
            s.outmap[y*16+x].tile    = c;
            s.outmap[y*16+x].palette = pal;
        }
}

static Level Fireman =
{
	3, // level number
	"Fireman",

	// screens:
	{
		{
		"##    ....      "   "                "
		"##    ....      "   "                "
		"##    ....      "   "                "
		"##    ....      "   "                "
		"##|#############"   "       v       v"
		"##|           ##"   "                "
		"##|           ##"   "                "
		"##|           ##"   "                "
		"#############|##"   "       v        "
		"##           |##"   "                "
		"##           |##"   "                "
		"##           |##"   "                "
		"################"   "                "
		"^^^^^^^^^^^^^^^^"   "                "
		"^^^^^^^^^^^^^^^^"   "                "
		"^^^^^^^^^^^^^^^^"   "                "
		,Right},
		{
		"..............##"   "     v          "
		"..............##"   "                "
		"      ........##"   "                "
		"      ........##"   "                "
		"#############|##"   "           v    "
		"      ....   |##"   "                "
		"        ..   |##"   "                "
		"        ..   |##"   "                "
		"|###############"   "      v         "
		"|               "   "                "
		"|               "   "                "
		"|               "   "                "
		"################"   "                "
		"^^^^^^^^^^^^^^^^"   "                "
		"^^^^^^^^^^^^^^^^"   "                "
		"^^^^^^^^^^^^^^^^"   "                "
		,Right},
		{
		"####.........|##"   "                "
		"####.........|##"   "                "
		"####.........|##"   "                "
		"####.........|##"   "    E           "
		"######....#|####"   "                "
		"####...... |  ##"   "                "
		"##........ |  ##"   "                "
		"##........ |  ##"   "                "
		".......... |  ##"   "                "
		".......... |  ##"   "                "
		"......#T.. |  ##"   "                "
		"......##.. |  ##"   "                "
		"####..##..##..##"   "       I        "
		"^^^^^^##^^^^^^^^"   "                "
		"^^^^^^##^^^^^^^^"   "     d          "
		"^^^^^^##^^^^^^^^"   "                "
		,Up},
		{
		"##.|..........##"   "                "
		"##.|..........##"   "                "
		"##.|..........##"   "                "
		"##.|..........##"   "                "
		"##............##"   "                "
		"##....#######|##"   "                "
		"##...........|##"   "                "
		"##...........|##"   "  e             "
		"###T.........|##"   "                "
		"####......#|####"   "                "
		"####.......|..##"   "   I            "
		"####.......|..##"   "    e           "
		"#####T.....|..##"   "                "
		"#############|##"   "                "
		"#############|##"   "     I          "
		"#############|##"   "                "
		,Up},
		{
		"##.............."   "                "
		"##.............."   "                "
		"##.............."   "                "
		"##.............."   "              PP"
		"##........######"   "                "
		"##........######"   "                "
		"##........######"   "                "
		"##........######"   "                "
		"##........######"   "                "
		"##.............."   "                "
		"##.|###T........"   "       I        "
		"##.|####..######"   "                "
		"##.|#####T######"   "       I        "
		"##.|############"   "                "
		"##.|############"   "         I      "
		"##.|############"   "                "
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"........####...."   "                "
		"........###>...."   "            ] - "
		"#>....####......"   "  ] -           "
		"........##....#."   "                "
		"........##......"   "                "
		">....#..##......"   " ] -            "
		"........####..##"   "                "
		".###...###....##"   "                "
		"........##....##"   "                "
		"........##....##"   "                "
		,Right},
		{
		"..............##"   "                "
		"..............##"   "                "
		"..............##"   "                "
		"..............##"   "                "
		"..............##"   "                "
		"..............##"   "                "
		"####..#T..##..##"   "                "
		"####..##..##..##"   "                "
		"..###T##..##..##"   "       I        "
		"..##########..##"   "                "
		"..######......##"   "     I          "
		"..######......##"   "                "
		"########......##"   "        eee     "
		"############.|##"   "                "
		"############.|##"   "                "
		"############.|##"   "                "
		,Down},
		{
		"############.|##"   "                "
		"############.|##"   "                "
		"####..######.|##"   "                "
		"####..######.|##"   "                "
		"####.........|##"   "                "
		"####.........|##"   "                "
		"############.|##"   "                "
		"####.........|##"   "                "
		"####..........##"   "                "
		"####..........##"   "                "
		"##............##"   "                "
		"##............##"   "                "
		"##......#T..#T##"   "                "
		"##.|##..########"   "                "
		"##.|##..########"   "      d  I   I  "
		"##.|############"   "                "
		,Down},
		{
		"##.|############"   "                "
		"##.|############"   "                "
		"##.|..##########"   "                "
		"##.|..##########"   "                "
		"##.............."   "                "
		"##.............."   "                "
		"##............=="   "                "
		"##...........|##"   "                "
		"##...........|.."   "                "
		"##...........|.."   "                "
		"##......#T...|.."   "                "
		"##......##..##.."   "                "
		"##..#T..##......"   "         I      "
		"##^^##^^##^^^^^^"   "                "
		"##^^##^^##^^^^^^"   "     I     d    "
		"##^^##^^##^^^^^^"   "                "
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"====......====.."   "                "
		"####.....|####.."   "                "
		".........|......"   "                "
		".........|......"   "                "
		"........##......"   "                "
		"................"   "                "
		"..............#T"   "                "
		"^^^^^^^^^^^^^^##"   "                "
		"^^^^^^^^^^^^^^##"   "     d         I"
		"^^^^^^^^^^^^^^##"   "                "
		,Right},
		{
		"###########|%%##"   "                "
		"###########|%%##"   "                "
		"########...|%%##"   "                "
		"########...|%%##"   "                "
		"...........|%%##"   "                "
		"...........|%%##"   "                "
		"...........|%%##"   "                "
		"...........|%%##"   "                "
		"...........|%%##"   "                "
		"...........|%%##"   "                "
		"..#T..######%%##"   "                "
		"..##........%%##"   "                "
		"..##............"   "   I            "
		"^^##^^^^^^^^^^^^"   "                "
		"^^##^^^^^^^^^^^^"   "    d           "
		"^^##^^^^^^^^^^^^"   "                "
		,Up},
		{
		"##  |     ..####"   "                "
		"##  |     ..  ##"   "                "
		"##  |     ..  ##"   "                "
		"##  |     ..  ##"   "                "
		"##  |     ..  ##"   "                "
		"##  %%%%%%%%| ##"   "                "
		"##  %%%%%%%%| ##"   "                "
		"##  ..      | ##"   "                "
		"##  ..      | ##"   "                "
		"###|%%%%%%%%%%##"   "                "
		"###|%%%%%%%%%%##"   "                "
		"## |        %%##"   "                "
		"## |        %%##"   "                "
		"###########|%%##"   "                "
		"###########|%%##"   "                "
		"###########|%%##"   "                "
		,Up},
		{
		"#..............."   "                "
		"#..............."   "                "
		"#.......        "   "                "
		"#.......        "   "                "
		"#.......      ##"   "                "
		"#.......      ##"   "                "
		"#.....      ####"   "                "
		"#.....      ####"   "                "
		"#.....    ######"   "                "
		"#.....    ####  "   "                "
		"#.....  ######  "   "              s "
		"#.....  ######  "   "                "
		"#...  ##########"   "                "
		"##..|#####./^^^^"   "                "
		"##..|#####//^^^^"   "                "
		"##..|#####^^^^^^"   "                "
		,Right},
		{
		"..............##"   "                "
		"..............##"   "                "
		"      ........##"   "                "
		"      ........##"   "             f  "
		"#############|##"   "                "
		"      ....   |##"   "                "
		"        ..   |##"   "                "
		"        ..   |##"   " f              "
		"|###############"   "                "
		"|               "   "                "
		"|               "   "                "
		"|               "   "               f"
		"################"   "                "
		"^^^^^^^^^^^^^^^^"   "                "
		"^^^^^^^^^^^^^^^^"   "                "
		"^^^^^^^^^^^^^^^^"   "                "
		,Right},
		{
		"..        ......"   "                "
		"..        ......"   "                "
		"          ......"   "                "
		"          ......"   "  P             "
		"  |         ...."   "                "
		"  |         ...."   "                "
		"  |             "   "                "
		"  |             "   "                "
		"  |             "   "                "
		"  |             "   "                "
		"  |             "   "                "
		"  |             "   "                "
		"####  ##  ##  ##"   "                "
		"^^^^^^^^^^^^^^##"   "                "
		"^^^^^^^^^^^^^^##"   "        d   d   "
		"^^^^^^^^^^^^^^##"   "        s       "
		,Right},
		{
		".............###"   "                "
		".............###"   "                "
		".............###"   "                "
		".............###"   "                "
		"..    .......###"   "                "
		"..    .......###"   "                "
		"        ......DD"   "              SS"
		"        ......DD"   "              SS"
		"          ....DD"   "              SS"
		"          ....DD"   "              SS"
		"          #T#T##"   "                "
		"          ######"   "                "
		"################"   "           I I  "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		,Shutter},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"################"   "                "
		"################"   "                "
		"                "   "X          .    "
		"                "   "X               "
		"                "   "X               "
		"                "   "X               "
		"################"   "                "
		"################"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"################"   "                "
		"################"   "                "
		"                "   "     .        . "
		"                "   "                "
		"                "   "                "
		"                "   "                "
		"################"   "                "
		"################"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"################"   "                "
		"################"   "                "
		"               D"   "      .        S"
		"               D"   "               S"
		"               D"   "               S"
		"               D"   "               S"
		"################"   "                "
		"################"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		,Shutter},
		{
		"################"   "                "
		"################"   "                "
		"#              #"   "                "
		"#              #"   "                "
		"#              #"   "                "
		"#              #"   "                "
		"               #"   "X               "
		"               #"   "X               "
		"               #"   "X               "
		"               #"   "X               "
		"#              #"   "                "
		"#              #"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		,End},
	},

	// palette
	{
		15, 48, 16,  6, 
		15, 27, 11,  0, 
		15, 39, 22,  6, 
		15, 38, 22,  6, 

		15, 44, 17, 15, 
		15, 32, 56, 15, 
		15, 39, 21, 15, 
		15, 48, 17, 15, 

		15, 48, 16,  6, 
		15, 32, 16,  0, 
		15, 39, 22,  6, 
		15, 48, 16, 22, 
	},

	// actives
	{
	},

	// enemies
	{
		Enemy( 0, 0x80,0xE0, 0x32 ),
		Enemy( 2, 0x78,0xC0, 0xF ),
		Enemy( 3, 0x38,0xA0, 0xF ),
		Enemy( 3, 0x58,0xE0, 0xF ),
		Enemy( 4, 0x98,0xE0, 0xF ),
		Enemy( 6, 0x78,0x80, 0xF ),
		Enemy( 6, 0x58,0xA0, 0xF ),
		Enemy( 7, 0x98,0xE0, 0xF ),
		Enemy( 7, 0xD8,0xE0, 0xF ),
		Enemy( 8, 0x98,0xC0, 0xF ),
		Enemy( 8, 0x58,0xE0, 0xF ),
		Enemy( 10, 0xD0,0x04, 0xE ),
		Enemy( 10, 0x38,0xC0, 0xF ),
		Enemy( 11, 0xB0,0xFF, 0xE ),
		Enemy( 15, 0xB8,0xC0, 0xF ),
		Enemy( 15, 0xD8,0xC0, 0xF ),
	},

	FiremanAscii, FiremanChar
};


