/*
ti pa

01.1       = air

3E.0 3F.0
20.0 28.0 = background: mountain

3C.2       = background: mountain edge (top left = sky)
3D.2       = background: mountain edge (top right = sky)

37.3       = background: fence

25.0       = background: hole in mountain (bottom right = darkness)
26.0       = background: hole in mountain (bottom left = darkness)
2C.0       = background: \ crack in mountain
0F.0       = background: / crack in mountain
34.0       = background: / crack in mountain (another look)
00.0       = background: darkness

35.0       = background: inside a tunnel

70.1       = horizontal steel bar (when rock below)
73.1       = horizontal steel bar (when air below)
67.1       = bridge foot (probably wall)

6E.1       = vertical steel bar (anchored at left side)
6F.1       = vertical steel bar
76.1       = vertical steel bar (anchored at right side)

71.3 72.3
69.3 6A.3 = wall
6B.3       = wall right edge
6D.3       = wall bottom edge

21.2       = lift path edge (*) (sky background)
22.2       = lift path      (-)
23.2       = lift path hole (/)

84.1       = spike up (sky bg)



*/
static void GutsmanAscii(Screen& s)
{
    for(unsigned y=0; y<16; ++y)
        for(unsigned x=0; x<16; ++x)
        {
            unsigned char c = s.outmap[y*16+x].tile;
            char ch = '?';
            /* Classify types.
             *
             *
             *
             */
            switch(c)
            {
                case 0x01:
                    ch = '.'; // sky
                    break;
                case 0x71: case 0x72:
                case 0x69: case 0x6A:
                case 0x6B: case 0x6D: // rock wall.
                case 0x70: case 0x73: // horizontal steel bar.
                case 0x51: case 0x52: case 0x53: // wily signature
                    ch = '#';
                    break;
                case 0x67: // bridge foot
                    ch = 'X';
                    break;
                case 0x6E: case 0x6F:
                case 0x76: // vertical steel bar.
                    ch = '&';
                    break;
                
                case 0x3C ... 0x3F:
                case 0x20: case 0x28:
                case 0x25: case 0x26:
                case 0x2C: case 0x0F: case 0x34:
                    ch = ' '; // mountain background
                    break;
                
                case 0x00: // dark background
                case 0x37: // fence
                case 0x35: // inside background
                    ch = ',';
                    break;
                
                case 0x10:
                    ch = Ascii_WilyDoor;
                    break;
                
                case 0x84:
                    ch = Ascii_Spike;
                    break;
                
                case 0x38:
                case 0x39:
                case 0x3A:
                case 0x3B:
                    ch = Ascii_Guts;
                    break;
                
                case 0x21: ch = '*'; break;
                case 0x22: ch = '-'; break;
                case 0x23: ch = '/'; break;

                // no ladders
            }
            
            s.ascii[y*32+x] = ch;
            s.ascii[y*32+x+16] = ' ';
            //printf("%02X",c);
        }
}

#define GutsmanChar  DummyChar

static Level Gutsman =
{
	5, // level number
	"Gutsman",

	// screens:
	{
		{
		"          ......"   "                "
		"          ......"   "                "
		"            ..  "   "                "
		"                "   "                "
		"                "   "                "
		"                "   "                "
		"                "   "                "
		"                "   "                "
		"              ##"   "                "
		"              ##"   "             m  "
		"   ,,,,     ####"   "                "
		"  ,,,,,,    ####"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		,Right},
		{
		"...        ....."   "                "
		"..          ...."   "                "
		"            ...."   "                "
		"            ...."   "                "
		"        ######.*"   "                "
		"        ######.."   "       m        "
		"    ##########.."   "                "
		"    ##########.."   "   m            "
		"##############.."   "                "
		"##############.."   "                "
		"##############.."   "                "
		"##############.."   "                "
		"##############.."   "                "
		"##############.."   "                "
		"##############.."   "                "
		"##############.."   "                "
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"----------------"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		".......*------/-"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"...............*"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"*..............."   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"------/-*......."   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"---/---/---/---/"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		,Right},
		{
		"................"   "                "
		"................"   "                "
		".............   "   "                "
		"............    "   "                "
		"............    "   "                "
		"............    "   "                "
		"...........     "   "                "
		"..........      "   "                "
		"..........      "   "                "
		"..........      "   "                "
		".........       "   "                "
		".........       "   "                "
		"---/-*... ######"   "                "
		"......... ######"   "                "
		"......... ######"   "                "
		"......... ######"   "                "
		,Right},
		{
		"          ......"   "                "
		"          ......"   "                "
		"            ..  "   "                "
		"                "   "                "
		"                "   "      y         "
		"                "   "           y    "
		"                "   "                "
		"                "   "                "
		"              ##"   "          y     "
		"              ##"   "                "
		"   ,,,,     ####"   "                "
		"  ,,,,,,    ####"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		,Right},
		{
		"...             "   "                "
		"..              "   "                "
		"                "   "                "
		"                "   "                "
		"      ##      ##"   "                "
		"      ##      ##"   "                "
		"    ######  ####"   "                "
		"    ######  ####"   "           E    "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		,Right},
		{
		"          ......"   "                "
		"          ......"   "                "
		"            ..  "   "                "
		"                "   "                "
		"                "   "                "
		"                "   "                "
		"                "   "                "
		"                "   "                "
		"              ##"   "            a   "
		"              ##"   "                "
		"   ,,,,     ####"   "                "
		"  ,,,,,,    ####"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		,Right},
		{
		"................"   "                "
		"................"   "                "
		"   ............."   "                "
		"   ............."   "                "
		"   ............."   "       a        "
		"   ............."   "                "
		"    ............"   "                "
		"    ............"   "                "
		"###########..##."   "                "
		"#,  .X...X...X.."   "                "
		"#,  .X...X...X.."   "                "
		"#,  .X...X...X.."   "                "
		"#,  .X...X...X.."   "                "
		"#,  .X...X...X.."   "                "
		"#,  .X...X...X.."   "                "
		"#,  .X...X...X.."   "                "
		,Right},
		{
		"..............&#"   "                "
		"..............&#"   "                "
		"..............&#"   "                "
		"..............&#"   "                "
		"..............&#"   "a            a  "
		"..............&#"   "                "
		"..............&#"   "                "
		"..............&#"   "                "
		"..##########..&#"   "                "
		"...X.X...##&..&#"   "                "
		"...X.X...##&&.&#"   "                "
		"...X.X...##&..&#"   "                "
		"...X.X...##&..&#"   "                "
		"^^^X^X^^^##&..&#"   "                "
		"###########&.&&#"   "                "
		"###########&..&#"   "                "
		,Down},
		{
		"#&............&#"   "                "
		"#&............&#"   "                "
		"#&............&#"   "                "
		"#&............&#"   "                "
		"#&............&#"   "                "
		"#&............&#"   "                "
		"#&....&&......&#"   "                "
		"##&&..&&......&#"   "                "
		"##&&..&&..&&..&#"   "                "
		"##&&..&&..&&..&#"   "                "
		"##&&..&&..&&..&#"   "                "
		"##&&..&&..&&..&#"   "                "
		"##&&..&&..&&..&#"   "                "
		"##&&..&&..&&..&#"   "                "
		"##&&..&&..&&..&#"   "                "
		"##&&..&&..&&..&#"   "              y "
		,Down},
		{
		"##&&..&&..&&..&#"   "                "
		"##&&..&&..&&..&#"   "                "
		"##&&..&&..&&..&#"   "                "
		"##&&..&&..&&..&#"   "                "
		"#&............&#"   "                "
		"#&............&#"   "           E    "
		"#&........&&..&#"   "                "
		"#&........&&..&#"   "  ee            "
		"##&&......&&..&#"   "                "
		"##&&......&&..&#"   "       p        "
		"##&&..&&......&#"   "                "
		"##&&^^&&......&#"   "                "
		"#####&&&..&&..&#"   "                "
		"#####&&&..&&..&#"   "                "
		"#####&&&..&&..&#"   "                "
		"#####&&&..&&..&#"   "              y "
		,Down},
		{
		"#####&&&..&&..&#"   "                "
		"#####&&&..&&..&#"   "                "
		"#&............&#"   "                "
		"#&............&#"   "                "
		"#&....&&......&#"   "                "
		"#&....&&......&#"   "   U            "
		"##&&..&&......&#"   "                "
		"##&&..&&......&#"   "                "
		"##&&..&&......&#"   "                "
		"##&&..&&......&#"   "                "
		"##&&..........&#"   "                "
		"##&&........^^&#"   "          e     "
		"##&&..##########"   "                "
		"##&&..##########"   "                "
		"##&&..##########"   "                "
		"##&&..##########"   "                "
		,Down},
		{
		"##&&............"   "                "
		"##&&............"   "                "
		"##&&............"   "                "
		"##&&............"   "                "
		"##&&.........   "   "                "
		"##&&........    "   "                "
		"####.......     "   "                "
		"####.......     "   "                "
		"########..      "   "                "
		"########..      "   "                "
		"############    "   "              t "
		"############    "   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		,Right},
		{
		".....        ###"   "                "
		"....         ###"   "                "
		"....         ###"   "                "
		"....         ###"   "                "
		"             ###"   "                "
		"             ###"   "                "
		"              DD"   "              SS"
		"              DD"   "              SS"
		"              DD"   "              SS"
		"              DD"   "              SS"
		"   ,,,,   ######"   "                "
		"  ,,,,,,  ######"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		,Shutter},
		{
		",,,,,,,,,,,,,,,,"   "                "
		",,,,,,,,,,,,,,,,"   "                "
		",,,,,,,,,,,,,,,,"   "                "
		",,,,,,,,,,,,,,,,"   "                "
		"################"   "                "
		"################"   "                "
		",,,,,,,,,,,,,,,,"   "X               "
		",,,,,,,,,,,,,,,,"   "X               "
		",,,,,,,,,,,,,,,,"   "X               "
		",,,,,,,,,,,,,,,,"   "X             m "
		"################"   "                "
		"################"   "                "
		",,,,,,,,,,,,,,,,"   "                "
		",,,,,,,,,,,,,,,,"   "                "
		",,,,,,,,,,,,,,,,"   "                "
		",,,,,,,,,,,,,,,,"   "                "
		,Right},
		{
		",,,,,,,,,,,,,,,,"   "                "
		",,,,,,,,,,,,,,,,"   "                "
		",,,,,,,,,,,,,,,,"   "                "
		",,,,,,,,,,,,,,,,"   "                "
		"################"   "                "
		"################"   "                "
		",,,,,,,,,,,,,,,,"   "                "
		",,,,,,,,,,,,,,,,"   "                "
		",,,,,,,,,,,,,,,,"   "                "
		",,,,,,,,,,,,,,,,"   "        m       "
		"################"   "                "
		"################"   "                "
		",,,,,,,,,,,,,,,,"   "                "
		",,,,,,,,,,,,,,,,"   "                "
		",,,,,,,,,,,,,,,,"   "                "
		",,,,,,,,,,,,,,,,"   "                "
		,Right},
		{
		",,,,,,,,,,,,,,,,"   "                "
		",,,,,,,,,,,,,,,,"   "                "
		",,,,,,,,,,,,,,,,"   "                "
		",,,,,,,,,,,,,,,,"   "                "
		"################"   "                "
		"################"   "                "
		",,,,,,,,,,,,,,,D"   "               S"
		",,,,,,,,,,,,,,,D"   "               S"
		",,,,,,,,,,,,,,,D"   "               S"
		",,,,,,,,,,,,,,,D"   "  m        m   S"
		"################"   "                "
		"################"   "                "
		",,,,,,,,,,,,,,,,"   "                "
		",,,,,,,,,,,,,,,,"   "                "
		",,,,,,,,,,,,,,,,"   "                "
		",,,,,,,,,,,,,,,,"   "                "
		,Shutter},
		{
		"&,,,,,,,,,,,,,,&"   "                "
		"&,,,,,,,,,,,,,,&"   "                "
		"&,,,,,,,,,,,,,,&"   "                "
		"&,,,,,,,,,,,,,,&"   "                "
		"&,,,,,,,,,,,,,,&"   "                "
		"&,,,,,,,,,,,,,,&"   "                "
		",,,,,,,,,,,,,,,&"   "X               "
		",,,,,,,,,,,,,,,&"   "X               "
		",,,,,,,,,,,,,,,&"   "X               "
		",,,,,,,,,,,,,,,&"   "X               "
		"##GGGG,,,,,,,,,&"   "                "
		"##GGGG,,,,,,,,,&"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		,End},
	},

	// palette
	{
		15, 38, 23,  7, 
		15, 48, 16,  2, 
		15, 38, 23,  2, 
		15, 55, 39, 23, 

		15, 44, 17, 15, 
		15, 32, 56, 15, 
		15, 48, 42, 15, 
		15, 48, 39, 15, 

		15, 48, 32, 16, 
		15, 48, 16,  2, 
		15, 38, 23,  2, 
		15, 55, 39, 23, 
	},

	// actives
	{
	},

	// enemies
	{
		Enemy( 0, 0xF8,0x80, 0x30 ),
		Enemy( 0, 0x80,0xFF, 0x2C ),
		Enemy( 1, 0xFC,0x80, 0x2F ),
		Enemy( 4, 0x58,0x80, 0x2F ),
		Enemy( 5, 0x01,0x08, 0xFF ),
		Enemy( 5, 0x40,0x0A, 0xFF ),
		Enemy( 5, 0x60,0x80, 0x30 ),
		Enemy( 6, 0x10,0x04, 0xFF ),
		Enemy( 6, 0x50,0x06, 0xFF ),
	},

	GutsmanAscii, GutsmanChar
};


