#ifndef bqtRockEditLEVElHH
#define bqtRockEditLEVElHH

#include <algorithm>

#include "defs.hh"
#include "atoms.hh"

struct Level
{
    unsigned LevelNumber;
    const char* Name;
    
    Screen rooms[MaxRooms];

    unsigned char Palette[16*3];
    
    Active actives[MaxActives];
    Enemy  enemies[MaxEnemies];

    /* Each level has different representations
     * in text form.
     */
    void (*CreateAscii)(Screen& screen);
    void (*CreateChar)(Screen& screen);
    
    void WritePalette(unsigned addr) const;
    void WriteActives(unsigned first_screen, unsigned addr, unsigned end_addr) const;
    void WriteEnemies(unsigned first_screen,
                      unsigned ptraddr, unsigned& addr, unsigned end_addr) const;
    void ReadPalette(unsigned addr);
    void ReadActives(unsigned first_screen, unsigned addr);
    void ReadEnemies(unsigned first_screen, unsigned ptraddr);
    
    void SortActives()
    {
        std::sort(actives, actives+MaxActives);
    }
    void SortEnemies()
    {
        std::sort(enemies, enemies+MaxEnemies);
    }
    

/*
Buzzers in Iceman level.
69
83
88
8B
A0
BA
D8
*/


    void AnalyzeObjects()
    {
        /* To be called after everything has been read from the ROM
         * and the ascii map has been generated, before dumping them
         * to output.
         */
        
        /* Delete actives that can be expressed on the map. */
        DeleteGutsBlockActives();
        
        /* Move the shutters from the active list to the object list. */
        MakeAsciiActives();
        
        /* Move movable enemies to the ascii map. */
        MakeAsciiEnemies();
        
        /* Last, sort the arrays. */
        SortActives();
        SortEnemies();
    }
    
    void ComposeObjects()
    {
        CreateGutsBlockActives();
        ReadAsciiActives();
        ReadAsciiEnemies();
        
        /* Last, sort the arrays. */
        SortActives();
        SortEnemies();
    }    
    
    void DeleteGutsBlockActives();
    void MakeAsciiActives();
    
    void MakeAsciiEnemies();

    void CreateGutsBlockActives();

    void ReadAsciiActives();
    void ReadAsciiEnemies();
    
    void AddActive(const Active& b)
    {
        for(unsigned a=0; a<MaxActives; ++a)
            if(actives[a].screen == 255)
            {
                actives[a] = b;
                return;
            }
        fprintf(stderr, "Couldn't add an active!\n");
    }
    
    void AddEnemy(const Enemy& b)
    {
        for(unsigned a=0; a<MaxEnemies; ++a)
            if(enemies[a].screen == 255)
            {
                enemies[a] = b;
                return;
            }
        fprintf(stderr, "Couldn't add an enemy!\n");
    }
};

struct LevelPairData
{
    Level& level1;
    Level& level2;
    
    unsigned BeginAddr;
};

void GenericAsciiMaker(Screen& s, int n,...);

#endif
